/*
 * gc global variable provides access to GUI Composer infrastructure components and project information.
 * For more information, please see the Working with Javascript guide in the online help.
 */
var gc = gc || {};
gc.services = gc.services || {};

/*
*  Boilerplate code for creating computed data bindings
*/
document.addEventListener('gc-databind-ready', function() 
{
    gc.databind.registry.bind('widget.logarithmicHor.checked','widget.efficiencyGraph.xAxisIsLogScale',null,function(value){return value;});
});

document.addEventListener("gc-nav-ready", function() {
    gc.nav.registerAction('saveGraphs', function() {
        let curveData=templateObj.efficiencyGraph._plotObj.getData();
        let parsedData='';
        
        console.log(curveData);
        let numOfCurves=curveData.length;

        let statusOk=Array(numOfCurves).fill(true);
        let row=0;
        
        parsedData+="\"sep=\t\"\n";
        
        // Loops for writing header labels
        for (let curve=0; curve<numOfCurves; curve++) {
            parsedData+=curveData[curve].label;
            if (curve==numOfCurves-1) {
                parsedData+='\n';
            } else {
                parsedData+='\t\t';
            }
        }
        
        // Vins
        for (let buck=1; buck<=templateObj.config.device.buckCount; buck++) {
            if (templateObj.config.BIoutData!==0 && templateObj.config.phases[buck-1]!=-1 && templateObj.BEnables[buck-1].checked) {
                if (templateObj.config.buckVinMasterFlag) {
                    parsedData+='Vin(mV)\t'+templateObj.config.Vin.toString()+'\t';
                } else {
                    parsedData+='Vin(mV)\t'+templateObj.config.BVin[buck-1].toString()+'\t';
                }
            }
        }
        for (let boost=1; boost<=templateObj.config.device.boostCount; boost++) {
            if (templateObj.config.BoostIoutData!==0 && templateObj.BoostEnables[boost-1].checked) {
                parsedData+='Vin(mV)\t'+templateObj.config.Vin.toString()+'\t';
            }
        }
        parsedData+='\n';
        
        // Vouts
        for (let buck=1; buck<=templateObj.config.device.buckCount; buck++) {
            if (templateObj.config.BIoutData!==0 && templateObj.config.phases[buck-1]!=-1 && templateObj.BEnables[buck-1].checked) {
                parsedData+='Vout(mV)\t'+templateObj.config.BVout[buck-1].toString()+'\t';
            }
        }
        for (let boost=1; boost<=templateObj.config.device.boostCount; boost++) {
            if (templateObj.config.BoostIoutData!==0 && templateObj.BoostEnables[boost-1].checked) {
                parsedData+='Vout(mV)\t'+templateObj.config.BoostVout[boost-1].toString()+'\t';
            }
        }
        parsedData+='\n';
        
                
        // FSWs
        for (let buck=1; buck<=templateObj.config.device.buckCount; buck++) {
            if (templateObj.config.BIoutData!==0 && templateObj.config.phases[buck-1]!=-1 && templateObj.BEnables[buck-1].checked) {
                parsedData+='FSW(MHz)\t'+templateObj.config.FSW[buck-1].toString()+'\t';
            }
        }
        for (let boost=1; boost<=templateObj.config.device.boostCount; boost++) {
            if (templateObj.config.BoostIoutData!==0 && templateObj.BoostEnables[boost-1].checked) {
                parsedData+='FSW(MHz)\t'+templateObj.config.BoostFSW[boost-1].toString()+'\t';
            }
        }
        parsedData+='\n';
        
        for (let curve=0; curve<numOfCurves; curve++) {
            parsedData+='Output current\tEfficiency';
            if (curve==numOfCurves-1) {
                parsedData+='\n';
            } else {
                parsedData+='\t';
            }
        }
        
        
        
        // Outer loop goes through the rows
        while (!statusOk.every(x=>!x)) {
            // Inner loop goes through the curves
            for (let curve=0; curve<numOfCurves; curve++) {
                if (row<curveData[curve].data.length) {
                    parsedData+=curveData[curve].data[row][0].toString()+'\t'+curveData[curve].data[row][1].toString();
                    if (curve==numOfCurves-1) {
                        parsedData+='\n';
                    } else {
                        parsedData+='\t';
                    }
                } else {
                    statusOk[curve]=false;
                    parsedData+='\t';
                    if (curve==numOfCurves-1) {
                        parsedData+='\n';
                    } else {
                        parsedData+='\t';
                    }
                }
            }
            row++;
        }
        console.log(parsedData);
        if (gc.desktop.isDesktop()) {
            gc.File.browseAndSave(parsedData,'efficiencyGraphs','*.csv',null,null);
        } else {
            window.saveAs(new Blob([parsedData], {type: "text/plain;charset=utf-8"}), "efficiencyGraphs.csv")
        }
    });
    
    gc.nav.registerAction('actionTPS6594LP8764GuiLink', function() {
        gc.desktop.openBrowserWindow('https://dev.ti.com/gallery/info/PMIC/Programmable-Processor-PMICs-GUI/');
    });
    
    gc.nav.registerAction('actionLP8756GuiLink', function() {
        gc.desktop.openBrowserWindow('https://www.ti.com/lit/zip/snvc202');
    });
    
    gc.nav.registerAction('action70GuiLink', function() {
        gc.desktop.openBrowserWindow('https://www.ti.com/lit/zip/snvc221');
    });
    
    gc.nav.registerAction('action58GuiLink', function() {
        gc.desktop.openBrowserWindow('https://www.ti.com/lit/zip/snvc189');
    });
});

var initComplete = false;
var templateObj;

let example = {
    'id':'deviceID',                            // Id of the device
    'name':'GPN',                               // Name of the device, will be shown above the efficiency graph and device selection box
    'minVin':0,                                 // Minimum input voltage, use an array if want to define input voltage for each buck separately
    'maxVin':0,                                 // Maximum input voltage, use an array if want to define input voltage for each buck separately
    'minBuckVout':[],                           // Minimum output voltage allowed, must be an array, each element of the array is one buck
    'maxBuckVout':[],                           // Maximum output voltage allowed, must be an array, each element of the array is one buck
    'maxBuckIout':[],                           // Single phase maximum currents for each rail, must be an array
    'buckCount':0,                              // Buck count
    'buckMinVoutVinDiff':0,                     // Minimum difference between VIN and VOUT for bucks
    'bucksPhaseConfigurable':true,              // True if phase configurations are available
    'fswCtrlPerPhase':true,                     // True if the switching frequency of the phases can be separately controlled
    'buckPfmPossible':true,                     // True if PFM mode available
    'fswOptions':[],                            // Array of the possible switching frequency options
    'buckInductor':[''],                        // Buck inductor array. If the device has bucks, this array must be the same size as the buckFswOptions. Use '' if the inductor details don't want to be shown.
    'buckInductorDCR':[[]],                     // Two dimensional array of inductor DCR. First index determines the buck and the second one determines the used FSW.
    'boostFswOptions':[],                       // Array of the possible switching frequency options for boosts
    'minBoostVout':[],                          // Minimum output voltage allowed, must be an array, each element of the array is one boost
    'maxBoostVout':[],                          // Maximum output voltage allowed, must be an array, each element of the array is one boost
    'maxBoostIout':[],                          // Maximum currents for each boost, must be an array
    'boostCount':0,                             // Boost count
    'boostInductor':[''],                       // Boost inductor array. If the device has boosts, this array must be the same size as the boostFswOptions. Use '' if the inductor details don't want to be shown.
    'boostInductorDCR':[[]],                    // Two dimensional array of inductor DCR. First index determines the buck and the second one determines the used FSW.
    'ldoMinVoutVinDiff':0,                      // Minimum difference between LDO VIN and VOUT
    'minLdoVin':0,                              // Minimum LDO input voltage. Must be an array where each element corresponds to one rail.
    'minLdoVout':0,                             // Minimum LDO output voltage. Must be an array where each element corresponds to one rail.
    'maxLdoVout':0,                             // Maximum LDO output voltage. Must be an array where each element corresponds to one rail.
    'maxLdoIout':0,                             // Maximum LDO output voltage. Must be an array where each element corresponds to one rail.
    'ldoCount':0,                               // LDO count
    'buckDataFile':'',                          // Filename of the buck efficiency data file
    'ldoDataFile':'',                           // Filename of the LDO efficiency data file. THIS HAS NOT BEEN IMPLEMENTED AND CAN BE LEFT EMPTY!!!
    'boostDataFile':'',                         // Filename of the boost efficiency data file
    'buckData':[{}],                            // Loaded buck efficiency data will be written to here. Do not edit this!
    'boostData':[{}],                           // Loaded boost efficiency data will be written to here. Do not edit this!
    'defaultTRes':0                             // The default thermal resistance value that will be used.
}

let TPS6594 = {
    'id':'deviceTPS6594',
    'name':'TPS6594-Q1',
    'minVin':2600,
    'maxVin':5500,
    'minBuckVout':[300,300,300,300,300],
    'maxBuckVout':[3340,3340,3340,3340],
    'maxBuckIout':[3500,3500,3500,4000,2000],
    'buckCount':5,
    'buckMinVoutVinDiff':700,
    'bucksPhaseConfigurable':true,
    'fswCtrlPerPhase':true,
    'buckPfmPossible':true,
    'fswOptions':[2.2,4.4],
    'buckInductor':['TFM322512ALMAR47MTAA','TFM322512ALMAR22MTAA'],
    'buckInductorDCR':[[16,6],[16,6],[16,6],[16,6],[16,6]],
    'boostFswOptions':0,
    'minBoostVout':0,
    'maxBoostVout':0,
    'maxBoostIout':0,
    'boostCount':0,
    'boostInductor':[''],
    'boostInductorDCR':[[]],
    'ldoMinVoutVinDiff':300,
    'minLdoVin':[1200,1200,1200,2200],
    'minLdoVout':[600,600,600,1200],
    'maxLdoVout':[3300,3300,3300,3300],
    'maxLdoIout':[500,500,500,300],
    'ldoCount':4,
    'buckDataFile':'TPS6594BuckData.json',
    'ldoDataFile':'',
    'boostDataFile':'',
    'buckData':[{}],
    'boostData':[{}],
    'defaultTRes':3.9
}

let LP8764 = {
    'id':'deviceLP8764',
    'name':'LP8764-Q1',
    'minVin':2600,
    'maxVin':5500,
    'minBuckVout':[300,300,300,300],
    'maxBuckVout':[3340,3340,3340,3340],
    'maxBuckIout':[5000,5000,5000,5000],
    'buckCount':4,
    'buckMinVoutVinDiff':700,
    'bucksPhaseConfigurable':true,
    'fswCtrlPerPhase':true,
    'buckPfmPossible':true,
    'fswOptions':[2.2,4.4,8.8],
    'buckInductor':['TFM322512ALMAR47MTAA','TFM322512ALMAR22MTAA','TFM322512ALMAR22MTAA'],
    'buckInductorDCR':[[16,6,6],[16,6,6],[16,6,6],[16,6,6]],
    'boostFswOptions':0,
    'minBoostVout':0,
    'maxBoostVout':0,
    'maxBoostIout':0,
    'boostCount':0,
    'boostInductor':[''],
    'boostInductorDCR':[[]],
    'ldoMinVoutVinDiff':0,
    'minLdoVin':0,
    'minLdoVout':0,
    'maxLdoVout':0,
    'maxLdoIout':0,
    'ldoCount':0,
    'buckDataFile':'LP8764BuckData.json',
    'boostDataFile':'',
    'ldoDataFile':'',
    'buckData':[{}],
    'boostData':[{}],
    'defaultTRes':6.0
}

let LP8756 = {
    'id':'deviceLP8756',
    'name':'LP8756/52-Q1',
    'minVin':2800,
    'maxVin':5500,
    'minBuckVout':[600,600,600,600],
    'maxBuckVout':[3360,3360,3360,3360],
    'maxBuckIout':[4000,4000,4000,4000],
    'buckCount':4,
    'buckMinVoutVinDiff':500,
    'bucksPhaseConfigurable':true,
    'fswCtrlPerPhase':false,
    'buckPfmPossible':true,
    'fswOptions':[2,3,4],
    'buckInductor':['DFE252012PD-R47M','DFE252012PD-R47M','DFE252012PD-R47M'],
    'buckInductorDCR':[[20,20,20],[20,20,20],[20,20,20],[20,20,20]],
    'boostFswOptions':0,
    'minBoostVout':0,
    'maxBoostVout':0,
    'maxBoostIout':0,
    'boostCount':0,
    'boostInductor':[''],
    'boostInductorDCR':[[]],
    'ldoMinVoutVinDiff':0,
    'minLdoVin':0,
    'minLdoVout':0,
    'maxLdoVout':0,
    'maxLdoIout':0,
    'ldoCount':0,
    'buckDataFile':'LP8756BuckData.json',
    'boostDataFile':'',
    'ldoDataFile':'',
    'buckData':[{}],
    'boostData':[{}],
    'defaultTRes':4.7
}

let LP8770 = {
    'id':'deviceLP87702',
    'name':'LP87702-Q1',
    'minVin':2800,
    'maxVin':5500,
    'minBuckVout':[600,600],
    'maxBuckVout':[3360,3360],
    'maxBuckIout':[3500,3500],
    'buckCount':2,
    'buckMinVoutVinDiff':1000,
    'bucksPhaseConfigurable':false,
    'fswCtrlPerPhase':false,
    'buckPfmPossible':true,
    'fswOptions':[2,3,4],
    'buckInductor':['DFE252012PD-R47M','DFE252012PD-R47M'],
    'buckInductorDCR':[[20,20,20],[20,20,20]],
    'boostFswOptions':[4],
    'minBoostVout':[4900],
    'maxBoostVout':[5200],
    'maxBoostIout':[600],
    'boostCount':1,
    'boostInductor':['DFE252012PD-1R0M'],
    'boostInductorDCR':[[40]],
    'ldoMinVoutVinDiff':0,
    'minLdoVin':0,
    'minLdoVout':0,
    'maxLdoVout':0,
    'maxLdoIout':0,
    'ldoCount':0,
    'buckDataFile':'LP8770BuckData.json',
    'boostDataFile':'LP8770BoostData.json',
    'ldoDataFile':'',
    'buckData':[{}],
    'boostData':[{}],
    'defaultTRes':5.6
}

let LP8758 = {
    'id':'deviceLP8758',
    'name':'LP8758',
    'minVin':2500,
    'maxVin':5500,
    'minBuckVout':[500,500,500,500],
    'maxBuckVout':[3360,3360,3360,3360],
    'maxBuckIout':[4000,4000,4000,4000],
    'buckCount':4,
    'buckMinVoutVinDiff':700,
    'bucksPhaseConfigurable':true,
    'fswCtrlPerPhase':false,
    'buckPfmPossible':true,
    'fswOptions':[3],
    'buckInductor':['DFE201610E-R47M'],
    'buckInductorDCR':[[26],[26],[26],[26]],
    'boostFswOptions':0,
    'minBoostVout':0,
    'maxBoostVout':0,
    'maxBoostIout':0,
    'boostCount':0,
    'boostInductor':[''],
    'boostInductorDCR':[[]],
    'ldoMinVoutVinDiff':0,
    'minLdoVin':0,
    'minLdoVout':0,
    'maxLdoVout':0,
    'maxLdoIout':0,
    'ldoCount':0,
    'buckDataFile':'LP8758BuckData.json',
    'boostDataFile':'',
    'ldoDataFile':'',
    'buckData':[{}],
    'boostData':[{}],
    'defaultTRes':8.5
}

// Have to check values
let TPS650330 = {
    'id':'deviceTPS650330',
    'name':'TPS650330-Q1',
    'minVin':[4000,3000,3000],
    'maxVin':[18300,5500,5500],
    'minBuckVout':[3800,1200,1200], // Real values would be [3000,900,900] but this is limited by the dataset
    'maxBuckVout':[3800,1200,1200], // Real values would be [4000,1900,1900] but this is limited by the dataset
    'maxBuckIout':[1500,1200,1200],
    'buckCount':3,
    'buckMinVoutVinDiff':500,
    'bucksPhaseConfigurable':false,
    'fswCtrlPerPhase':false,
    'buckPfmPossible':true,
    'fswOptions':[2.3],
    'buckInductor':[''],
    'buckInductorDCR':[[110],[60],[60]],
    'boostFswOptions':0,
    'minBoostVout':0,
    'maxBoostVout':0,
    'maxBoostIout':0,
    'boostCount':0,
    'boostInductor':[''],
    'boostInductorDCR':[[]],
    'ldoMinVoutVinDiff':300,
    'minLdoVin':[3000],
    'minLdoVout':[2700],
    'maxLdoVout':[3300],
    'maxLdoIout':[300],
    'ldoCount':1,
    'buckDataFile':'TPS650330BuckData.json',
    'boostDataFile':'',
    'ldoDataFile':'',
    'buckData':[{}],
    'boostData':[{}],
    'defaultTRes':10.5
}

let LP8733 = {
    'id':'deviceLP8733',
    'name':'LP8733/32-Q1',
    'minVin':2800,
    'maxVin':5500,
    'minBuckVout':[700,700],
    'maxBuckVout':[3360,3360],
    'maxBuckIout':[3000,3000],
    'buckCount':2,
    'buckMinVoutVinDiff':800,
    'bucksPhaseConfigurable':false,
    'fswCtrlPerPhase':false,
    'buckPfmPossible':true,
    'fswOptions':[2,3,4],
    'buckInductor':['DFE252012PD-R47M','DFE252012PD-R47M','DFE252012PD-R47M'],
    'buckInductorDCR':[[20,20,20],[20,20,20]],
    'boostFswOptions':0,
    'minBoostVout':0,
    'maxBoostVout':0,
    'maxBoostIout':0,
    'boostCount':0,
    'boostInductor':0,
    'boostInductorDCR':0,
    'ldoMinVoutVinDiff':0,
    'minLdoVin':[2500, 2500],
    'minLdoVout':[800, 800],
    'maxLdoVout':[3300, 3300],
    'maxLdoIout':[300, 300],
    'ldoCount':2,
    'buckDataFile':'LP8733EffData.json',
    'boostDataFile':'',
    'ldoDataFile':'',
    'buckData':[{}],
    'boostData':[{}],
    'defaultTRes':8.9
}


let DEVICES=[example,TPS6594,LP8764,LP8756,LP8770,LP8758,TPS650330,LP8733]; // Add additional devices to this array.
let PHASECONFIGS=[[4,-1,-1,-1,1],
                    [3,-1,-1,1,1],
                    [2,-1,1,1,1],
                    [1,1,1,1,1],
                    [2,-1,2,-1,1]];

// Wait for DOMContentLoaded event before trying to access the application template
var init = function() {
    templateObj = document.querySelector('#template_obj');
    
    // The master config is here. Everything should be accessed through this.
    templateObj.config = {
        'device':example,
        'Vin':3300,
        'buckVinMasterFlag':true, // Use the master VIN if true
        'BVin':[0,0,0,0,0],
        'BVout':[0,0,0,0,0],
        'BIout':[0,0,0,0,0],
        'BoostVout':[0,0,0,0,0],
        'BoostIout':[0,0,0,0,0],
        'BEffData':[[],[],[],[],[]],
        'BIoutData':[[],[],[],[],[]],
        'BoostEffData':[[],[],[],[],[]],
        'BoostIoutData':[[],[],[],[],[]],
        'FSW':[2.2,2.2,2.2,2.2,2.2],
        'BoostFSW':[2,2,2,2,2],
        'FPWM':[1,1,1,1,1],
        'LDOVinMasterFlag':true, // Use the master VIN if true
        'LDOVin':[3300,3300,3300,3300],
        'LDOVout':[0,0,0,0],
        'LDOIout':[0,0,0,0],
        'phaseconfig':4, // 1, 2, 3, 4 and 5 (4-ph, 3+1, 2+1+1, 1+1+1+1 and 2+2)
        'phases':[1,1,1,1,1], // -1 if not existing or slave
        'BPin':[0.0,0.0,0.0,0.0,0.0],
        'BPout':[0.0,0.0,0.0,0.0,0.0],
        'BPdiss':[0.0,0.0,0.0,0.0,0.0],
        'BEff':[0.0,0.0,0.0,0.0,0.0],
        'BInductorLoss':[0,0,0,0,0],
        'BoostPin':[0.0,0.0,0.0,0.0,0.0],
        'BoostPout':[0.0,0.0,0.0,0.0,0.0],
        'BoostPdiss':[0.0,0.0,0.0,0.0,0.0],
        'BoostEff':[0.0,0.0,0.0,0.0,0.0],
        'BoostInductorLoss':[0],
        'LDOPin':[0.0,0.0,0.0,0.0,0.0],
        'LDOPout':[0.0,0.0,0.0,0.0,0.0],
        'LDOPdiss':[0.0,0.0,0.0,0.0,0.0],
        'LDOEff':[0.0,0.0,0.0,0.0,0.0],
        'systemPin':0.0,
        'systemPout':0.0,
        'systemEff':0.0,
        'systemPdiss':0.0,
        'icPdiss':0.0,
        'systemIin':0.0,
        'totalInductorLoss':0.0,
        'totalBuckInductorLoss':0.0,
        'totalBoostInductorLoss':0.0,
        'junctionTemp':0.0
    };

    // Wait for the template to fire a dom-change event to indicate that it has been 'stamped'
    // before trying to access components in the application.
	templateObj.addEventListener('dom-change',function(){
	    if (initComplete) return;
	    this.async(function(){
    	    initComplete = true;
    	    //initializeWidgets();
    	    console.log("Application template has been stamped.");
  	        // Now that the template has been stamped, you can use 'automatic node finding' $ syntax to access widgets.
  	        // e.g. to access a widget with an id of 'widget_id' you can use templateObj.$.widgetId
            
            //let inputvoltage=templateObj.$.inputVoltageDBX;
            templateObj.deviceDBX=document.getElementById('deviceDBX');
            templateObj.vinInputBox=document.getElementById('vinInputBox');
            templateObj.forceMultiphaseDBX=document.getElementById('forceMultiphaseDBX');
            templateObj.phaseConfigurationDBX=document.getElementById('phaseConfigurationDBX');
            templateObj.tempSlider=document.getElementById('boardTempSlider');
            templateObj.thermalResistanceInput=document.getElementById('thermalResistanceInput');
            
            templateObj.BVoutBX=[];
            templateObj.BIoutBX=[];
            templateObj.FswDBX=[];
            templateObj.FpwmDBX=[];
            templateObj.BEnables=[];
            templateObj.BVinBX=[];
            for (let i = 1; i<=5; i++) {
                templateObj.BVoutBX.push(document.getElementById('B'+(i).toString()+'VoutInputBox'));
                templateObj.BIoutBX.push(document.getElementById('B'+(i).toString()+'IoutInputBox'));
                templateObj.FswDBX.push(document.getElementById('B'+(i).toString()+'switchingFrequencyDBX'));
                templateObj.FpwmDBX.push(document.getElementById('B'+(i).toString()+'forcePwmDBX'));
                templateObj.BEnables.push(document.getElementById('B'+(i).toString()+'Enable'));
                templateObj.BVinBX.push(document.getElementById('buck'+(i).toString()+'VinInputBox'));
                templateObj.BEnables[i-1].addEventListener('changed', function(e){enableRail('BUCK',i);parseConfig('BUCK');});
                templateObj.BVoutBX[i-1].addEventListener('changed', function(e){incrementCheck(i,e);parseConfig('BUCK');});
                templateObj.BIoutBX[i-1].addEventListener('changed', function(e){parseConfig('BUCK');});
                templateObj.FswDBX[i-1].addEventListener('selected', function(e){parseConfig('BUCK');setInductorText();});
                templateObj.FpwmDBX[i-1].addEventListener('selected', function(e){parseConfig('BUCK');});
                templateObj.BVinBX[i-1].addEventListener('changed', e=>parseConfig('BUCK'));
            }
            
            templateObj.BoostVoutBX=[];
            templateObj.BoostIoutBX=[];
            templateObj.BoostFswDBX=[];
            templateObj.BoostEnables=[];
            for (let i = 1; i<=1; i++) {
                templateObj.BoostVoutBX.push(document.getElementById('boost'+(i).toString()+'VoutInputBox'));
                templateObj.BoostIoutBX.push(document.getElementById('boost'+(i).toString()+'IoutInputBox'));
                templateObj.BoostFswDBX.push(document.getElementById('boost'+(i).toString()+'SwitchingFrequencyDBX'));
                templateObj.BoostEnables.push(document.getElementById('boost'+(i).toString()+'Enable'));
                templateObj.BoostEnables[i-1].addEventListener('changed', function(e){enableRail('BOOST',i);parseConfig('BOOST');});
                templateObj.BoostVoutBX[i-1].addEventListener('changed', function(e){parseConfig('BOOST');});
                templateObj.BoostIoutBX[i-1].addEventListener('changed', function(e){parseConfig('BOOST');});
                templateObj.BoostFswDBX[i-1].addEventListener('selected', function(e){parseConfig('BOOST');setInductorText();});
            }
            
            templateObj.masterFSW=document.getElementById('masterSwitchingFrequencyDBX');
            
            templateObj.LDOVinBX=[];
            templateObj.LDOVoutBX=[];
            templateObj.LDOIoutBX=[];
            templateObj.LDOEnables=[];
            for (let i = 1; i<=4; i++) {
                templateObj.LDOVinBX.push(document.getElementById('ldo'+(i).toString()+'VinInputBox'));
                templateObj.LDOVoutBX.push(document.getElementById('LDO'+(i).toString()+'VoutInputBox'));
                templateObj.LDOIoutBX.push(document.getElementById('LDO'+(i).toString()+'IoutInputBox'));
                templateObj.LDOEnables.push(document.getElementById('LDO'+(i).toString()+'Enable'));
                templateObj.LDOEnables[i-1].addEventListener('changed', function(e){enableRail('LDO',i,templateObj.LDOEnables[i-1].checked);parseConfig('LDO');});
                templateObj.LDOVinBX[i-1].addEventListener('changed', function(e){checkMaxVout(templateObj.LDOVinBX[i-1].value);parseConfig('LDO');});
                templateObj.LDOVoutBX[i-1].addEventListener('changed', function(e){parseConfig('LDO');});
                templateObj.LDOIoutBX[i-1].addEventListener('changed', function(e){parseConfig('LDO');});
            }
            templateObj.ldoIndividualVin=document.getElementById('ldoVinUseIndividual');
            templateObj.ldoIndividualVin.addEventListener('changed', function(e){hideLdoVins(templateObj.ldoIndividualVin.checked);parseConfig('LDO');});
            
            // Indicator widgets
            templateObj.efficiencyGraph=document.getElementById('efficiencyGraph');
            templateObj.efficiencyGraph._redrawOnOptionsChange=false;
            templateObj.efficiencyGraph._redrawOonValuesChange=false;
            
            // Override for the GC teams implementation for window resizing.
            templateObj.efficiencyGraph.onResize=function() {
                templateObj.efficiencyGraph._plotObj.resize();
                updateGraphs();
            }
            
            // Warning showed that functions should not be created in loops so here the loops have been unrolled. Beautiful isn't it? :)
            templateObj.deviceDBX.addEventListener('selected-value-changed', function(e){
                changeDevice(DEVICES[deviceDBX.selectedValue]);
                changePhaseConfig(parseInt(phaseConfigurationDBX.selectedValue));
                checkMaxVout(templateObj.vinInputBox.value);
                parseConfig('ALL');
                setInductorText();
            });
            templateObj.vinInputBox.addEventListener('changed', function(e){
                checkMaxVout(templateObj.vinInputBox.value);
                parseConfig('ALL');});
            templateObj.tempSlider.addEventListener('changed',function(e) {updateTables('THERMAL');});
            templateObj.thermalResistanceInput.addEventListener('changed',function(e) {updateTables('THERMAL');});
            templateObj.masterFSW.addEventListener('selected', function(e){parseConfig('BUCK');setInductorText();});
            templateObj.phaseConfigurationDBX.addEventListener('selected', function(e){changePhaseConfig(parseInt(phaseConfigurationDBX.selectedValue)); parseConfig('BUCK');});
            //templateObj.forceMultiphaseDBX.addEventListener('selected-value-changed', function(e){parseConfig('BUCK'); console.log("fmp change")});
	         
	        templateObj.logHor=document.getElementById('logarithmicHor');
	        templateObj.logHor.addEventListener('changed',function(e){updateGraphs();});
  	        loadData();
  	        
  	        // Initialization
            changeDevice(DEVICES[deviceDBX.selectedValue]);
            changePhaseConfig(parseInt(phaseConfigurationDBX.selectedValue));
            checkMaxVout(templateObj.vinInputBox.value);
            parseConfig('ALL');
	    },1);
	});
};

templateObj = document.querySelector('#template_obj');
if (templateObj) {
    init();
} else {
    document.addEventListener('DOMContentLoaded',init.bind(this));
}

// Reads the efficiency data to the device objects.
function loadData() {
    for (let i=0; i<DEVICES.length; i++) {
        let buckPath='app/data/'+DEVICES[i].buckDataFile;
        let boostPath='app/data/'+DEVICES[i].boostDataFile;
        let ldoPath='app/data/'+DEVICES[i].ldoDataFile;
        if (buckPath) {
            gc.fileCache.readJsonFile(buckPath).then(fileContent => {
                // If data is available, add the device to the selection.
                if (fileContent) {
                    DEVICES[i].buckData=fileContent;
                    templateObj.deviceDBX.labels+='|'+DEVICES[i].name;
                    templateObj.deviceDBX.values+='|'+i.toString();
                }
            });
        }
        if (boostPath) {
            gc.fileCache.readJsonFile(boostPath).then(fileContent => {
                // If data is available, add the device to the selection.
                if (fileContent) {
                    DEVICES[i].boostData=fileContent;
                }
            });
        }
    }
}

function setInductorText() {
    let indName='';
    for (let rail=0; rail<templateObj.config.device.buckCount+templateObj.config.device.boostCount; rail++) {
        if (rail<templateObj.config.device.buckCount) {
            if (templateObj.config.device.fswCtrlPerPhase) {
                indName=templateObj.config.device.buckInductor[templateObj.FswDBX[rail].selectedIndex];
            } else {
                indName=templateObj.config.device.buckInductor[templateObj.masterFSW.selectedIndex];
            }
            if (indName!='') {
                document.getElementById("B"+(rail+1).toString()+"Inductor").label="Inductor: "+indName;
                document.getElementById("B"+(rail+1).toString()+"Inductor").style.display='flex';
            } else {
                document.getElementById("B"+(rail+1).toString()+"Inductor").style.display='none';
            }
        } else {
            indName=templateObj.config.device.boostInductor[templateObj.BoostFswDBX[rail-templateObj.config.device.buckCount].selectedIndex];
            if (indName!='') {
                document.getElementById("boost"+(rail-templateObj.config.device.buckCount+1).toString()+"Inductor").label="Inductor: "+indName;
                document.getElementById("boost"+(rail-templateObj.config.device.buckCount+1).toString()+"Inductor").style.display='flex';
            } else {
                document.getElementById("boost"+(rail-templateObj.config.device.buckCount+1).toString()+"Inductor").style.display='none';
            }
        }
    }
}

function hideLdoVins(truefalse) {
   if (truefalse) {
        templateObj.config.LDOVinMasterFlag=false;
        for (let i=0; i<templateObj.config.device.ldoCount; i++) {
            document.getElementById('ldo'+(i+1).toString()+'VinContainer').style.display='flex';
            if (templateObj.config.buckVinMasterFlag) {
                templateObj.LDOVinBX[i].value=templateObj.config.Vin;
            } else {
                templateObj.LDOVinBX.value=templateObj.LDOVinBX[i].minValue;
            }
        }
    } else {
        templateObj.config.LDOVinMasterFlag=true;
        for (let i=0; i<templateObj.config.device.ldoCount; i++) {
            document.getElementById('ldo'+(i+1).toString()+'VinContainer').style.display='none';
        }
    }
}

function checkMaxVout(vin) {
    let minInputOutputDiff=templateObj.config.device.buckMinVoutVinDiff;
    let maxOutput=0;
    let railVout=0;
    for (let i=1;i<=templateObj.config.device.buckCount;i++) {
        if (templateObj.config.buckVinMasterFlag) {
            maxOutput=vin-minInputOutputDiff;
        } else {
            maxOutput=templateObj.BVinBX[i-1].value-minInputOutputDiff
        }
        railVout=templateObj.config.device.maxBuckVout[i-1];
        if (maxOutput > railVout) {
            maxOutput=railVout;
        } else if (maxOutput < 0) {
            console.log("error with max output voltage for buck");
            maxOutput=templateObj.config.device.maxBuckVout;
        }
        templateObj.BVoutBX[i-1].maxValue=maxOutput;
        // Check the set output voltage isn't higher than the allowed. This can
        // happen when lowering input voltage.
        if (templateObj.BVoutBX[i-1].value>templateObj.BVoutBX[i-1].maxValue) {
            templateObj.BVoutBX[i-1].value=templateObj.BVoutBX[i-1].maxValue;
        } else if (templateObj.BVoutBX[i-1].value<templateObj.BVoutBX[i-1].minValue) {
            templateObj.BVoutBX[i-1].value=templateObj.BVoutBX[i-1].minValue;
        }
    }
    
    minInputOutputDiff=templateObj.config.device.ldoMinVoutVinDiff;
    for (let i=1;i<=templateObj.config.device.ldoCount;i++) {
        if (templateObj.config.LDOVinMasterFlag) {
            maxOutput=vin-minInputOutputDiff;
        } else {
            maxOutput=templateObj.LDOVinBX[i-1].value-minInputOutputDiff;
        }
        railVout=templateObj.config.device.maxLdoVout[i-1];
        if (maxOutput > railVout) {
            maxOutput=railVout;
        } else if (maxOutput < 0) {
            console.log("error with max output voltage for ldo");
            maxOutput=templateObj.config.device.maxLdoVout;
        }
        templateObj.LDOVoutBX[i-1].maxValue=maxOutput;
        if (templateObj.LDOVoutBX[i-1].value>templateObj.LDOVoutBX[i-1].maxValue) {
            templateObj.LDOVoutBX[i-1].value=templateObj.LDOVoutBX[i-1].maxValue;
        } else if (templateObj.LDOVoutBX[i-1].value<templateObj.LDOVoutBX[i-1].monValue) {
            templateObj.LDOVoutBX[i-1].value=templateObj.LDOVoutBX[i-1].minValue;
        }
    }
}

function incrementCheck(rail,event) {
    let setValue=event.detail.newValue;
    // old value if we need a direction checks
    //let oldValue=event.detail.oldValue;
    if (setValue <= 600) {
        templateObj.BVoutBX[rail-1].increment=20;
    } else if (setValue <= 1100) {
        templateObj.BVoutBX[rail-1].increment=5;
    } else if (setValue <= 1660) {
        templateObj.BVoutBX[rail-1].increment=10;
    } else {
        templateObj.BVoutBX[rail-1].increment=20;
    }
}

function enableRail(buckorldo,rail) {
    if (buckorldo=='BUCK') {
        if (templateObj.BEnables[rail-1].checked) {
            document.getElementById('b'+rail.toString()+'EnableContainer').style.display='flex';
        } else {
            document.getElementById('b'+rail.toString()+'EnableContainer').style.display='none';
        }
    } else if (buckorldo=='LDO') {
        if (templateObj.LDOEnables[rail-1].checked) {
            document.getElementById('ldo'+rail.toString()+'EnableContainer').style.display='flex';
        } else {
            document.getElementById('ldo'+rail.toString()+'EnableContainer').style.display='none';
        }
    } else if (buckorldo=='BOOST') {
        if (templateObj.BoostEnables[rail-1].checked) {
            document.getElementById('boost'+rail.toString()+'EnableContainer').style.display='flex';
        } else {
            document.getElementById('boost'+rail.toString()+'EnableContainer').style.display='none';
        }
    } else {
        console.log("Unknown buckorldo parameter in enableRail-function"); 
    }
}

function changePhaseConfig(config) {
    if (templateObj.config.device.bucksPhaseConfigurable) {
        templateObj.config.phaseconfig=parseInt(phaseConfigurationDBX.selectedValue);
        templateObj.config.phases=PHASECONFIGS[config-1];
        document.getElementById("phaseConfigContainer").style.display='flex';
    } else {
        templateObj.config.phaseconfig=parseInt(4);
        templateObj.config.phases=PHASECONFIGS[3];
        document.getElementById("phaseConfigContainer").style.display='none';
    }
    for (let i=0;i<templateObj.BEnables.length;i++) {
        if (i<templateObj.config.device.buckCount) {
            if (templateObj.config.phases[i]!=-1) {
                document.getElementById('B'+(i+1).toString()+'Container').style.display='';
            } else {
                document.getElementById('B'+(i+1).toString()+'Container').style.display='none';
            }
        } else {
            document.getElementById('B'+(i+1).toString()+'Container').style.display='none';
        }
    }
    setMaxCurrents();
}

function setMaxCurrents() {
    // This function assumes that the multiphase config current max is 
    // defined by maximum of phase 1 multiplied by the phase count.
    switch(templateObj.config.phaseconfig) {
        case 1:
            templateObj.BIoutBX[0].maxValue=templateObj.config.device.maxBuckIout[0]*4;
            break;
        case 2:
            templateObj.BIoutBX[0].maxValue=templateObj.config.device.maxBuckIout[0]*3;
            templateObj.BIoutBX[3].maxValue=templateObj.config.device.maxBuckIout[0];
            break;
        case 3:
            templateObj.BIoutBX[0].maxValue=templateObj.config.device.maxBuckIout[0]*2;
            templateObj.BIoutBX[2].maxValue=templateObj.config.device.maxBuckIout[0];
            templateObj.BIoutBX[3].maxValue=templateObj.config.device.maxBuckIout[0];
            break;
        case 4:
            templateObj.BIoutBX[0].maxValue=templateObj.config.device.maxBuckIout[0];
            templateObj.BIoutBX[1].maxValue=templateObj.config.device.maxBuckIout[1];
            templateObj.BIoutBX[2].maxValue=templateObj.config.device.maxBuckIout[2];
            templateObj.BIoutBX[3].maxValue=templateObj.config.device.maxBuckIout[3];
            break;
        case 5:
            templateObj.BIoutBX[0].maxValue=templateObj.config.device.maxBuckIout[0]*2;
            templateObj.BIoutBX[2].maxValue=templateObj.config.device.maxBuckIout[0]*2;
            break;
        default:
            console.log("unknown phase config");
            break;
            
    }
    
    // This is for the fifth buck. Always set since it is never part of the multiphase config.
    templateObj.BIoutBX[4].maxValue=templateObj.config.device.maxBuckIout[4];
    
    for (let buck=1; buck<=templateObj.config.device.buckCount; buck++) {
        document.getElementById("B"+buck.toString()+"Enable").label="Buck"+buck.toString()+" ("+(templateObj.BIoutBX[buck-1].maxValue/1000).toFixed(1)+" A) Enable";
        // If set output current value is higher than after the change, set the
        // value to max current.
        if (templateObj.BIoutBX[buck-1].value>templateObj.BIoutBX[buck-1].maxValue) {
            templateObj.BIoutBX[buck-1].value=templateObj.BIoutBX[buck-1].maxValue;
        }
    }
    
    for (let boost=1; boost<=templateObj.config.device.boostCount; boost++) {
        templateObj.BoostIoutBX[boost-1].maxValue=templateObj.config.device.maxBoostIout[boost-1];
        document.getElementById("boost"+boost.toString()+"Enable").label="Boost"+boost.toString()+" ("+(templateObj.BoostIoutBX[boost-1].maxValue/1000).toFixed(1)+" A) Enable";
    }
}

// Cuts always the last character
function formatArrayToCharSeparatedString(array,separator) {
    let string='';
    for (let i=0; i<array.length; i++) {
        string+=array[i].toString()+separator;
        if (i==array.length-1) string=string.slice(0,-1);
    }
    return string;
}

function changeDevice(setDevice) {
    templateObj.config.device=setDevice;
    document.getElementById("thermalResistanceInput").value=templateObj.config.device.defaultTRes;
    // Iterates all the buck widgets and hides the extra ones and shows the available EVEN IF the phase config is wrong.
    // They are reiterated again to match phase configuration.
    for (let i=0;i<templateObj.config.device.buckCount;i++) {
        // IF VIN specification is an array, 
        if (Array.isArray(templateObj.config.device.minVin) && Array.isArray(templateObj.config.device.maxVin)) {
            document.getElementById('vinContainer').style.display='none';
            document.getElementById('buck'+(i+1).toString()+'VinContainer').style.display='flex';
            templateObj.config.buckVinMasterFlag=false;
            templateObj.BVinBX[i].minValue=templateObj.config.device.minVin[i];
            templateObj.BVinBX[i].maxValue=templateObj.config.device.maxVin[i];
            // Must use LDO individual VINs as well
            templateObj.ldoIndividualVin.checked=true;
            templateObj.ldoIndividualVin.style.display='none';
        } else {
            document.getElementById('vinContainer').style.display='flex';
            document.getElementById('buck'+(i+1).toString()+'VinContainer').style.display='none';
            templateObj.config.buckVinMasterFlag=true;
            templateObj.vinInputBox.minValue=templateObj.config.device.minVin;
            templateObj.vinInputBox.maxValue=templateObj.config.device.maxVin;
            //templateObj.ldoIndividualVin.checked=false;
            templateObj.ldoIndividualVin.style.display='flex';
        }

        // Set FSW options to the phase individual dropboxes
        templateObj.FswDBX[i].values=formatArrayToCharSeparatedString(templateObj.config.device.fswOptions,'|');
        templateObj.FswDBX[i].labels=formatArrayToCharSeparatedString(templateObj.config.device.fswOptions,' MHz|');

        // Check that if the value is undefined, set it to max value
        if (!templateObj.config.device.fswOptions.includes(templateObj.FswDBX[i].selectedValue)) {
            templateObj.FswDBX[i].selectedIndex=templateObj.config.device.fswOptions.length-1;
            templateObj.FswDBX[i].selectedValue=templateObj.config.device.fswOptions[templateObj.config.device.fswOptions.length-1];
        }
        
        if (!templateObj.config.device.buckPfmPossible) {
            document.getElementById('b'+(i+1).toString()+'FpwmContainer').style.display='none';
            document.getElementById('B'+(i+1).toString()+'forcePwmDBX').selectedValue=1;
        } else {
            document.getElementById('b'+(i+1).toString()+'FpwmContainer').style.display='';
        }
        
        // Minimum output voltage values for bucks
        templateObj.BVoutBX[i].minValue=templateObj.config.device.minBuckVout[i];
        
        // Maximum output voltage values for bucks
        templateObj.BVoutBX[i].maxValue=templateObj.config.device.maxBuckVout[i];
    }
    
    for (let i=0;i<templateObj.BoostEnables.length;i++) {
        if (i<templateObj.config.device.boostCount) {
            // Show boosts
            //document.getElementById('boost'+(i+1).toString()+'Container').classList.remove("hidden");
            document.getElementById('boost'+(i+1).toString()+'Container').style.display='flex';
            
            // Set FSW options to the phase individual dropboxes
            templateObj.BoostFswDBX[i].values=formatArrayToCharSeparatedString(templateObj.config.device.boostFswOptions,'|');
            templateObj.BoostFswDBX[i].labels=formatArrayToCharSeparatedString(templateObj.config.device.boostFswOptions,' MHz|');
            // Check that if the value is undefined, set it to max value
            if (!templateObj.BoostFswDBX[i].selectedValue) {
                templateObj.BoostFswDBX[i].selectedValue=templateObj.config.device.boostfswOptions[templateObj.config.device.boostFswOptions.length-1];
                templateObj.BoostFswDBX[i].selectedIndex=templateObj.config.device.boostfswOptions.length-1;
            }
            
            // Minimum output voltage values for boosts
            templateObj.BoostVoutBX[i].minValue=templateObj.config.device.minBoostVout[i];
            
            // Maximum output voltage values for boosts
            templateObj.BoostVoutBX[i].maxValue=templateObj.config.device.maxBoostVout[i];
        } else {
            // Hide extra boosts
            document.getElementById('boost'+(i+1).toString()+'Container').style.display='none';
        }
    }
    
    
    // Check if phases can use separate switching frequency
    if (templateObj.config.device.fswCtrlPerPhase) {
        for (let i=0;i<templateObj.BEnables.length;i++) {
            document.getElementById('b'+(i+1).toString()+'FswContainer').style.display='flex';
        }
        document.getElementById('masterFswContainer').style.display='none';
    } else {
        for (let i=0;i<templateObj.BEnables.length;i++) {
            document.getElementById('b'+(i+1).toString()+'FswContainer').style.display='none';
        }
        // No need to check the option values since the values have been checked already.
        templateObj.masterFSW.values=templateObj.FswDBX[0].values;
        templateObj.masterFSW.labels=templateObj.FswDBX[0].labels;
        document.getElementById('masterFswContainer').style.display='flex';
            
        // Check if the FSW is within the allowed values.
        if (!templateObj.config.device.fswOptions.includes(templateObj.masterFSW.selectedValue)) {
            templateObj.masterFSW.selectedIndex=templateObj.config.device.fswOptions.length-1;
            templateObj.masterFSW.selectedValue=templateObj.config.device.fswOptions[templateObj.config.device.fswOptions.length-1];
        }
    }
    
    if (templateObj.config.device.ldoCount===0) {
        document.getElementById('ldoSettingTab').style.display='none';
        document.getElementById('ldoTabButton').style.display='none';
        if (document.getElementById('tab_2')) document.getElementById('tab_2').style.display='none'; // this is the LDO table tab, ID digged by inspecting elements
    } else {
        document.getElementById('ldoSettingTab').style.display='';
        document.getElementById('ldoTabButton').style.display='';
        document.getElementById('tab_2').style.display=''; // this is the LDO table tab, ID digged by inspecting elements
        for (let i=0;i<templateObj.LDOEnables.length;i++) {
            if (i<templateObj.config.device.ldoCount) {
                templateObj.LDOVinBX[i].minValue=templateObj.config.device.minLdoVin[i];
                
                // Minimum output voltage values for bucks
                templateObj.LDOVoutBX[i].minValue=templateObj.config.device.minLdoVout[i];
                
                // Maximum output voltage values for bucks
                templateObj.LDOVoutBX[i].maxValue=templateObj.config.device.maxLdoVout[i];
                
                // Maximum output current values for bucks
                templateObj.LDOIoutBX[i].maxValue=templateObj.config.device.maxLdoIout[i];
                document.getElementById("LDO"+(i+1).toString()+"Enable").label="LDO"+(i+1).toString()+" ("+(templateObj.LDOIoutBX[i].maxValue/1000).toFixed(1)+" A) Enable";
                document.getElementById('LDO'+(i+1).toString()+'Container').style.display='flex';
            } else {
                document.getElementById('LDO'+(i+1).toString()+'Container').style.display='none';
            }
        }
    }
    setMaxCurrents();
    document.getElementById("efficiencyDevice").label=templateObj.config.device.name+" Efficiency";
}

function searchData(buckorldo,rail) {
    // We need to interpolate VIN, VOUT and form efficiency curve. After that interpolate IOUT from the efficiency.
    let isBuck=buckorldo=='BUCK'
    let isBoost=buckorldo=='BOOST'
    if (isBuck || isBoost) {
        let data={};
        
        // Look for closest input voltages.
        let interpolationData = '';
        if (isBuck) {
            interpolationData = templateObj.config.device.buckData.filter(function (element) {
                return element.FSW==templateObj.config.FSW[rail-1] &&
                element.FPWM==templateObj.config.FPWM[rail-1] &&
                element.phases==templateObj.config.phases[rail-1];
            });
        } else {
            interpolationData = templateObj.config.device.boostData.filter(function (element) {
                return element.FSW==templateObj.config.BoostFSW[rail-1]
            });
        }
        let vin=0;
        if (isBoost || templateObj.config.buckVinMasterFlag) {
            vin=templateObj.config.Vin;
        } else {
            vin=templateObj.config.BVin[rail-1];
        }
        let closestVins=findTwoClosestOneVariable(interpolationData,vin,'VIN');
        // If input voltage is exact match we don't have to interpolate VIN
        if (closestVins.CLOSE_POS_INDEX==closestVins.CLOSE_NEG_INDEX) {
            data=getGraph(rail,vin,buckorldo.toLowerCase());
        } else {
            // Get the closest VINs and interpolate the VOUT for both input voltages
            if (closestVins.CLOSE_NEG_INDEX==-1 || closestVins.CLOSE_POS_INDEX==-1) {
                data={'Iout':0,'Eff':0};
            } else if (isBoost && (interpolationData[closestVins.CLOSE_NEG_INDEX].Vout>templateObj.config.device.maxBoostVout[rail-1] || interpolationData[closestVins.CLOSE_NEG_INDEX].Vout<templateObj.config.device.minBoostVout[rail-1])) {
                data={'Iout':0,'Eff':0};
            } else if (isBuck && (interpolationData[closestVins.CLOSE_NEG_INDEX].Vout>templateObj.config.device.maxBuckVout[rail-1] || interpolationData[closestVins.CLOSE_NEG_INDEX].Vout<templateObj.config.device.minBuckVout[rail-1])) {
                data={'Iout':0,'Eff':0};
            } else {
                let closestNeg=interpolationData[closestVins.CLOSE_NEG_INDEX].VIN;
                let closestPos=interpolationData[closestVins.CLOSE_POS_INDEX].VIN;
                let approx3v3=getGraph(rail,closestNeg,buckorldo.toLowerCase()); // Efficiency curve for lower input voltage interpolated
                let approx5v=getGraph(rail,closestPos,buckorldo.toLowerCase()); // Efficiency curve for higher input voltage interpolated
                data=interpolateEfficiencyGraph(approx3v3, approx5v, 'VIN', vin); // Final interpolation
            }
        }
        if (isBuck) {
            templateObj.config.BEffData[rail-1]=data.Eff;
            templateObj.config.BIoutData[rail-1]=data.Iout;
            if (data.Iout==0 || data.Eff==0) {
                document.getElementById("B"+rail.toString()+"Enable").childNodes[1].childNodes[0].childNodes[1].style.backgroundColor="red"
            } else {
                document.getElementById("B"+rail.toString()+"Enable").childNodes[1].childNodes[0].childNodes[1].style.backgroundColor="#3f51b5"
            }
        } else {
            templateObj.config.BoostEffData[rail-1]=data.Eff;
            templateObj.config.BoostIoutData[rail-1]=data.Iout;
            if (data.Iout==0 || data.Eff==0) {
                document.getElementById("boost"+rail.toString()+"Enable").childNodes[1].childNodes[0].childNodes[1].style.backgroundColor="red"
            } else {
                document.getElementById("boost"+rail.toString()+"Enable").childNodes[1].childNodes[0].childNodes[1].style.backgroundColor="#3f51b5"
            }
        }
        interpolateDotCurrentEfficiency(rail,buckorldo);
    } else { // LDO if we would have data
        console.log("Search for LDO data here");
    }
}

function parseConfig(buckorldo) {
    templateObj.config.Vin=parseInt(templateObj.vinInputBox.value);
    if (buckorldo=='BUCK' || buckorldo=='ALL') {
        // Read all buck dropboxes
        for (let i = 0; i<templateObj.config.device.buckCount; i++) {
            if (templateObj.BEnables[i].checked && templateObj.config.phases[i]!=-1) {
                templateObj.config.BVin[i]=parseInt(templateObj.BVinBX[i].value);
                templateObj.config.BVout[i]=parseInt(templateObj.BVoutBX[i].value);
                templateObj.config.BIout[i]=parseInt(templateObj.BIoutBX[i].value);
                templateObj.config.FPWM[i]=parseInt(templateObj.FpwmDBX[i].selectedValue);
                // If using per phase controlled switching frequency
                if (templateObj.config.device.fswCtrlPerPhase) {
                    templateObj.config.FSW[i]=parseFloat(templateObj.FswDBX[i].selectedValue);   
                } else {
                    templateObj.config.FSW[i]=parseFloat(templateObj.masterFSW.selectedValue);
                }
                searchData('BUCK',i+1);
            }
        }
    }
    
    if (buckorldo=='BOOST' || buckorldo=='ALL') {
        for (let i = 0; i<templateObj.config.device.boostCount; i++) {
            if (templateObj.BoostEnables[i].checked) {
                templateObj.config.BoostVout[i]=parseInt(templateObj.BoostVoutBX[i].value);
                templateObj.config.BoostIout[i]=parseInt(templateObj.BoostIoutBX[i].value);
                templateObj.config.BoostFSW[i]=parseFloat(templateObj.BoostFswDBX[i].selectedValue);   
                searchData('BOOST',i+1);
            }
        }
    }
    
    // Not an ELSE branch since with ALL parameter we must be able to read everything.
    if ((buckorldo=='LDO' || buckorldo=='ALL')) {
        for (let i = 1; i<=templateObj.config.device.ldoCount; i++) {
            templateObj.config.LDOVin[i-1]=templateObj.LDOVinBX[i-1].value;
            templateObj.config.LDOVout[i-1]=templateObj.LDOVoutBX[i-1].value;
            templateObj.config.LDOIout[i-1]=templateObj.LDOIoutBX[i-1].value;
        }
    }
    updateGraphs();
    calculateResults(buckorldo);
    updateTables(buckorldo);
}